
from .Question import * 

from .exceptions import * 

class YesNoQuestion(Question):
    
    question_type = "yesno" 

    def __init__(self, msg, yes_points, no_points):
        self.msg = msg
        self.y = yes_points 
        self.n = no_points

    def ask(self):
        return self.msg + " (yes/no)"

    def getPoints(self):
        if self.getAnswer() in ["yes", "y", "Y", "Yes"]:
            return self.y
        elif self.getAnswer() in ["no", "n", "N", "No"]:
            return self.n
        else:
            raise WrongAnswerException()

    def getMessage(self):
        return self.msg

    def getYesPoints(self):
        return self.y

    def getNoPoints(self):
        return self.n





from .Question import * 

from .exceptions import * 

class MultiChoiceQuestion(Question):
    
    question_type = "multi_choice" 

    def __init__(self, msg):
        self.msg = msg
        self.choices = []

    def ask(self):
        s = self.msg + " (Enter number between 1 and %d)" % len(self.choices)

    def getPoints(self):
        ans = int(self.getAnswer()) - 1
        if ans < 0 or ans >= len(self.choices):
            raise WrongAnswerException()
        else:
            return self.choices[ans][1]

    def addChoice(self, choice, points):
        self.choices.append((choice, points))

    def getChoices(self):
        return self.choices

    def getMessage(self):
        return self.msg 




from .Question import * 

from .exceptions import * 

class ScoreQuestion(Question):
    
    question_type = "score" 

    def __init__(self, msg, min_score, max_score, points_per_score):
        self.msg = msg
        self.min_score = min_score 
        self.max_score = max_score
        self.points_per_score = points_per_score

    def ask(self):
        return self.msg + (" (%d-%d)" % (self.min_score, self.max_score))

    def getPoints(self):
        ans = int(self.getAnswer())
        if ans < self.min_score or ans > self.max_score:
            raise WrongAnswerException()
        else:
            return ans* self.points_per_score

    def getMessage(self):
        return self.msg

    def getScoreRange(self):
        return (self.min_score, self.max_score)

    def getPointsPerScore(self):
        return self.points_per_score




from .RankingSystem import * 

class StandardInputSingleCandidateRankingSystem(RankingSystem):
    
    def setup(self, **kwargs):
        pass

    def scoreCandidates(self):
        ss = self.getScoringSystem()
        for q in ss.getQuestions():
            if q.getType() == "multi_choice":
                print(q.ask())
                for i in range(len(q.getChoices())):
                    print("%d: %s" % (i+1, q.getChoices()[i][0]))
                q.answer(input("Answer: "))
            else:
                q.answer(input(q.ask() + " "))
        return ss.getScore()





class Question(object):
    
    """
    This has to be defined differently for each question type 
    """
    question_type = "base" 

    def ask(self):
        """
        Should return message for a given question
        """
        pass

    def getPoints(self):
        """
        Returns nuber which represents number of points for a given answer to specific question
        """
        pass

    def answer(self, ans):
        self.answer = ans 

    def getAnswer(self):
        return self.answer 
    
    def getType(self):
        return self.question_type




import json 

from .YesNoQuestion import * 
from .ScoreQuestion import * 
from .MultiChoiceQuestion import * 

class ScoringSystem(object):
    
    def __init__(self, name=""):
        self.questions = [] 
        self.name = name

    def addQuestion(self, q):
        self.questions.append(q)

    def getScore(self):
        score = 0 
        for q in self.questions:
            score = score + q.getPoints()
        return score

    def getQuestions(self):
        return self.questions

    def makeQuestionsDict(self):
        data = {}
        data["questions"] = [] 
        for q in self.getQuestions():
            Q = {}
            Q["type"] = q.getType()
            if Q["type"] == "yesno":
                Q["message"] = q.getMessage()
                Q["yes_points"] = q.getYesPoints()
                Q["no_points"] = q.getNoPoints()
            elif Q["type"] == "score":
                Q["message"] = q.getMessage()
                (Q["min_score"], Q["max_score"]) = q.getScoreRange()
                Q["points_per_score"] = q.getPointsPerScore()
            elif Q["type"] == "multi_choice":
                Q["message"] = q.getMessage()
                Q["choices"] = []
                for c in q.getChoices():
                    C = {}
                    C["choice"] = c[0]
                    C["points"] = c[1]
                    Q["choices"].append(C)
            data["questions"].append(Q)
        return data

    def dictToQuestion(q):
        if q["type"] == "yesno":
            return YesNoQuestion(q["message"], int(q["yes_points"]), int(q["no_points"]))
        elif q["type"] == "score":
            return ScoreQuestion(q["message"], int(q["min_score"]), int(q["max_score"]), float(q["points_per_score"]))
        elif q["type"] == "multi_choice":
            mq = MultiChoiceQuestion(q["message"])
            for c in q["choices"]:
                mq.addChoice(c["choice"], int(c["points"]))
            return mq
        

    def saveQuestionsToJSON(self, path):
            f = open(path, "w")
            q = self.makeQuestionsDict()
            q["name"] = self.name
            f.write(json.dumps(q, indent=4, sort_keys=True))
            f.close()

    def loadQuestionsFromJSON(path):
        f = open(path)
        data = json.loads(f.read())
        name = data.get("name", "")
        ss = ScoringSystem(name)
        for q in data["questions"]:
            ss.addQuestion(ScoringSystem.dictToQuestion(q))
        f.close()
        return ss

    def saveAnswersToJSON(self, path):
        data = self.makeQuestionsDict()
        data["name"] = self.name
        for i in range(len(data["questions"])):
            for question in self.getQuestions():
                if question.ask() == ScoringSystem.dictToQuestion(data["questions"][i]).ask():
                    data["questions"][i]["answer"] = question.getAnswer()
        f = open(path, "w")
        f.write(json.dumps(data, indent=4, sort_keys=True))
        f.close()

    def loadAnswersFromJSON(path):
        ss = ScoringSystem.loadQuestionsFromJSON(path)
        f = open(path)
        data = json.loads(f.read())
        s = ScoringSystem()
        for q in ss.getQuestions():
            for question in data["questions"]:
                if ScoringSystem.dictToQuestion(question).ask() == q.ask():
                    q.answer(question["answer"])
                    s.addQuestion(q)
        return s




class RankingSystem():
    
    def __init__(self, scoring_system, **kwargs):
        self.scoring_system = scoring_system
        self.setup(**kwargs)

    def setup(self, **kwargs):
        """
        This has to be implemented to setup specific ranking system
        """
        pass

    def scoreCandidates(self):
        """
        This has to be implemented for every specific ranking system. 

        This method is aimed to provide answers to questions for scoring system
        """
        pass

    def getScoringSystem(self):
        return self.scoring_system



from distutils.core import setup
setup(
  name = 'Marina-decide',
  packages = ['Marina-decide'], # this must be the same as the name above
  version = '0.13',
  description = 'Decision making tool',
  package_dir = {'Marina-decide': 'lib'},
  author = 'Stefan Nožinić',
  author_email = 'stefan@lugons.org',
  url = 'https://github.com/fantastic001/Marina', # use the URL to the github repo
  download_url = 'https://github.com/fantastic001/Marina/tarball/0.13', 
  keywords = ['decision-making', 'scoring', 'organization', 'lifehack'], # arbitrary keywords
  classifiers = [],
  scripts = ["marina-build", "marina-decide"],
)



from .exceptions import * 
from .MultiChoiceQuestion import * 
from .Question import * 
from .ScoreQuestion import * 
from .ScoringSystem import * 
from .YesNoQuestion import * 
from .RankingSystem import * 
from .StandardInputSingleCandidateRankingSystem import * 

